﻿using System;
using System.Reflection;
using Commander.API;

namespace Commander.Commands.BuiltIn
{
	public class CommanderCommand : ICommand
	{
		private readonly AssemblyName assemblyInfo = Assembly.GetExecutingAssembly().GetName();

		public string Execute(ICommandAccess access, string[] arguments)
		{
			if (arguments.Length < 1)
			{
				return Help();
			}

			foreach (var arg in arguments)
			{
				if (arg == "version")
				{
					return VersionInfo();
				}
				
				if (arg == "about")
				{
					return About();
				}
			}

			return Help();
		}

		private string About()
		{
			return VersionInfo() + Environment.NewLine + 
				    "Written by Marcus Grenängen aKa SneWs " + Environment.NewLine + 
					"Send emails to: (mailto:marcus@grenangen.se)" + Environment.NewLine +
					"For details and latest version, visit http://grenangen.se or https://github.com/SneWs/Commander";
		}

		private string VersionInfo()
		{
			return String.Format("Commander version {0}", Version);
		}

		private string Help()
		{
			return String.Format("{0}", VersionInfo()) + Environment.NewLine +
			       "Usage: " + Identifier + " -version|-about[-ui]" + Environment.NewLine +
				   "\t-version \t: Shows version information" + Environment.NewLine +
				   "\t-about \t: Shows information about the application" + Environment.NewLine +
				   "\t-ui \t: (Optional) Add this argument if you want the information displayed using a Windows UI instead of the command prompt." + Environment.NewLine;
		}


		private string Version
		{
			get { return assemblyInfo.Version.ToString(); }
		}

		public string Identifier
		{
			get { return "commander"; }
		}
	}
}
