﻿using System;
using System.Collections.Generic;
using System.Text;
using Commander.API;

namespace Commander.Commands.BuiltIn
{
	public class HistoryCommand : ICommand
	{

		public string Execute(ICommandAccess access, string[] arguments)
		{
			if (arguments.Length < 1)
			{
				return List(access.History);
			}

			for (int i = 0; i < arguments.Length; i++)
			{
				string arg = arguments[i];
				if (arg == "search" || arg == "s")
				{
					string criteria = (i + 1 < arguments.Length ? arguments[i + 1] : null);
					return String.IsNullOrEmpty(criteria) ? Help() : Search(criteria, access.History);
				}

				if (arg == "list" || arg == "l")
				{
					return List(access.History);
				}

				if (arg == "help" || arg == "h")
				{
					return Help();
				}
			}

			return List(access.History);
		}

		private static string Search(string criteria, ICommandHistory history)
		{
			StringBuilder sb = new StringBuilder();
			history.GetMatching(criteria).ForEach(record => sb.AppendLine(record));
			sb.AppendLine("");
			return sb.ToString();
		}

		private static string List(ICommandHistory history)
		{
			StringBuilder sb = new StringBuilder();
			List<string> records = history.Records;

			records.ForEach(record => sb.AppendLine(record));
			sb.AppendLine("");
			sb.AppendLine(String.Format("{0} entries in history", records.Count));
			
			return sb.ToString();
		}

		private string Help()
		{
			return "Usage: " + Identifier + " -search|-list" + Environment.NewLine +
			       "\t-help|-h \t: Shows this help." + Environment.NewLine +
				   "\t-search|-s \t: Searches the history and prints the list of hits." + Environment.NewLine +
			       "\t-list|-l \t: Dumps the history to screen" + Environment.NewLine;
		}

		public string Identifier
		{
			get { return "history"; }
		}
	}
}
