﻿using System.Collections.Generic;
using System.Windows.Forms;
using Commander.API;
using Commander.UI.Presenters;
using Commander.UI.Views;
using NUnit.Framework;
using Rhino.Mocks;

namespace Commander.Tests.System.UI
{
	[TestFixture]
	public class AutoCompletePresenterTests
	{
		private MockRepository _mocks;
		private ICommandHistory _commandHistory;
		private Control _attachedTo;
		private AutoCompleteView _view;
		private AutoCompletePresenter _presenter;

		[SetUp]
		public void SetUp()
		{
			_mocks = new MockRepository();

			_view = new AutoCompleteView();
			_attachedTo = new Control();
			_commandHistory = _mocks.DynamicMock<ICommandHistory>();
			_presenter = new AutoCompletePresenter(_view, _attachedTo, _commandHistory);

			_presenter.Init();
		}

		[TearDown]
		public void TearDown()
		{
			
		}


		[Test]
		public void PresentsAllItemsInHistoryWhenShown()
		{
			_commandHistory.Expect(x => x.Records).Return(new List<string> { "cls", "dir", "C:" });
			_mocks.ReplayAll();

			_presenter.ShowItems();
			
			Assert.AreEqual(3, _presenter.View.ItemsList.Items.Count);
			Assert.AreEqual("C:", _presenter.View.ItemsList.Items[0]);
			Assert.AreEqual("dir", _presenter.View.ItemsList.Items[1]);
			Assert.AreEqual("cls", _presenter.View.ItemsList.Items[2]);
			Assert.AreEqual("3 items", _presenter.View.BottomCountLabel.Text);
		}

		[Test]
		public void PresenterSendsItemClickedEventWhenItemClicked()
		{
			_commandHistory.Expect(x => x.Records).Return(new List<string> { "cls", "dir", "C:" });
			_mocks.ReplayAll();

			bool didSendProperEvent = false;
			_presenter.ShowItems();
			_presenter.ItemClicked += (sender, e) => didSendProperEvent = (e.Command == "dir");
			_presenter.View.ItemsList.SelectedIndex = 1;
			_presenter.SelectCurrentItem();

			Assert.IsTrue(didSendProperEvent);
		}
	}
}
