﻿using System.Collections.Generic;
using System.Windows.Forms;
using Commander.Core;
using Commander.UI.Presenters;
using Commander.UI.Views;
using NUnit.Framework;
using Rhino.Mocks;

namespace Commander.Tests.System.UI
{
	[TestFixture]
	public class ClipboardHistoryPresenterTests
	{
		private MockRepository _mocks;
		private Form _owner;
		private IClipboardActions _output;
		private IClipboardHistory _clipboardHistory;
		private ClipboardHistoryView _view;
		private ClipboardHistoryPresenter _presenter;

		[SetUp]
		public void SetUp()
		{
			_mocks = new MockRepository();

			_view = new ClipboardHistoryView();
			_clipboardHistory = _mocks.DynamicMock<IClipboardHistory>();
			_output = _mocks.DynamicMock<IClipboardActions>();
			_owner = new Form();
			_presenter = new ClipboardHistoryPresenter(_view, _clipboardHistory, _output, _owner);
			_presenter.Init();
		}

		[TearDown]
		public void TearDown()
		{
			
		}


		[Test]
		public void PresenterShowsAllClipboardHistory()
		{
			_clipboardHistory.Expect(x => x.Records).Return(new List<string> { "historyOne", "historyTwo" });
			_mocks.ReplayAll();

			_presenter.ShowItems();

			Assert.AreEqual("historyOne", _presenter.View.ClipboardHistoryList.Items[0]);
			Assert.AreEqual("historyTwo", _presenter.View.ClipboardHistoryList.Items[1]);
		}

		[Test]
		public void PresenterShowsAllClipboardHistoryInclCurrentClipboardContent()
		{
			_clipboardHistory.Expect(x => x.Records).Return(new List<string> { "historyOne", "historyTwo" });
			_clipboardHistory.Expect(x => x.CurrentClipboardContent).Return("current");
			_mocks.ReplayAll();

			_presenter.ShowItems();

			Assert.AreEqual("current", _presenter.View.ClipboardHistoryList.Items[0]);
			Assert.AreEqual("historyOne", _presenter.View.ClipboardHistoryList.Items[1]);
			Assert.AreEqual("historyTwo", _presenter.View.ClipboardHistoryList.Items[2]);
		}
	}
}
