﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Commander.Core;
using Commander.UI;

namespace Commander.Dialogs
{
	public partial class SettingsDialog : CommanderDialog
	{
		public SettingsDialog()
		{
			InitializeComponent();
		}


		public void ApplyTabSettings(TabSettings settings)
		{
			TextFont = settings.Font;
			BackgroundColor = settings.BackgroundColor;
			PromptColor = settings.PromptColor;
			TextColor = settings.TextColor;
		}


		private void OnFontLabelClicked(object sender, EventArgs e)
		{
			var lbl = (Label)sender;
			using (var dlg = new FontDialog())
			{
				dlg.Font = lbl.Font;
				if (dlg.ShowDialog(this) == DialogResult.OK)
				{
					lbl.Font = dlg.Font;
					lbl.Text = dlg.Font.Name + String.Format(", {0}pt", dlg.Font.SizeInPoints);
				}
			}
		}

		private void OnColorPanelClicked(object sender, EventArgs e)
		{
			var p = (Panel)sender;
			using (var dlg = new ColorDialog())
			{
				dlg.Color = p.BackColor;
				if (dlg.ShowDialog(this) == DialogResult.OK)
				{
					p.BackColor = dlg.Color;
				}
			}
		}

		private void OnSaveButtonClicked(object sender, EventArgs e)
		{
			DialogResult = DialogResult.OK;
			Close();
		}

		private void OnRestoreDefaultLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			ApplyTabSettings(new TabSettings());
		}


		#region Properties
		public Font TextFont
		{
			get { return lblFont.Font; }
			set
			{
				lblFont.Font = value;
				lblFont.Text = value.Name + String.Format(", {0}pt", value.SizeInPoints);
			}
		}

		public Color BackgroundColor
		{
			get { return pBgColor.BackColor; }
			set { pBgColor.BackColor = value; }
		}

		public Color TextColor
		{
			get { return pTextColor.BackColor; }
			set { pTextColor.BackColor = value; }
		}

		public Color PromptColor
		{
			get { return pPromptColor.BackColor; }
			set { pPromptColor.BackColor = value; }
		}
		#endregion
	}
}
