﻿using System.Collections.Generic;
using System.Threading;
using Commander.API;
using Commander.Core.Command;
using NUnit.Framework;
using Rhino.Mocks;

namespace Commander.Core.Tests.Command
{
	[TestFixture]
	public class CommandHostTests
	{
		private MockRepository _mocks;
		private CommandHost _commandHost;
		private ICommandManager _commandManager;
		private ICommandLoader _commandLoader;

		[SetUp]
		public void SetUp()
		{
			_mocks = new MockRepository();

			_commandManager = _mocks.StrictMock<ICommandManager>();
			_commandLoader = _mocks.StrictMock<ICommandLoader>();
			_commandHost = new CommandHost(_commandManager, _commandLoader);
		}

		[TearDown]
		public void TeadDown()
		{
			
		}


		[Test]
		public void CommandHostLoadsAllAssemblies()
		{
			_commandLoader.Expect(x => x.LoadAll()).Return(new List<ICommand>());
			_mocks.ReplayAll();

			var resetEvent = new ManualResetEvent(false);
			_commandHost.CommandsLoaded += (sender, e) => resetEvent.Set();
			_commandHost.Start();

			resetEvent.WaitOne(500);

			_mocks.Verify(_commandLoader);
		}

		[Test]
		[Timeout(1000)]
		public void CommandHostRegistersAllLoadedCommands()
		{
			ICommand cmdOne = _mocks.Stub<ICommand>();
			cmdOne.Expect(x => x.Identifier).Return("One");
			ICommand cmdTwo = _mocks.Stub<ICommand>();
			cmdTwo.Expect(x => x.Identifier).Return("Two");

			_commandLoader.Expect(x => x.LoadAll()).Return(new List<ICommand> { cmdOne, cmdTwo });
			_commandManager.Expect(x => x.RegisterHandler(cmdOne));
			_commandManager.Expect(x => x.RegisterHandler(cmdTwo));
			_mocks.ReplayAll();

			var resetEvent = new ManualResetEvent(false);
			_commandHost.CommandsLoaded += (sender, e) => resetEvent.Set();
			_commandHost.Start();

			resetEvent.WaitOne(250);

			Assert.AreEqual(2, _commandHost.LoadedCommands.Count);
			_mocks.Verify(_commandManager);
		}
	}
}
