﻿using System.Collections.Generic;
using System.IO;
using System.Reflection;
using Commander.API;
using Commander.Core.Command;
using NUnit.Framework;
using Rhino.Mocks;

namespace Commander.Core.Tests.Command
{
	[TestFixture]
	public class CommandLoaderTests
	{
		private MockRepository _mocks;
		private CommandLoader _loader;
		private DirectoryInfo _searchPath;
		private ICommandAssemblyLoader _commandLoader;

		[SetUp]
		public void SetUp()
		{
			_mocks = new MockRepository();

			_searchPath = new DirectoryInfo(Path.Combine(Path.GetTempPath(), "CmdLoaderTests"));
			_searchPath.Create();

			_commandLoader = _mocks.DynamicMock<ICommandAssemblyLoader>();
			_loader = new CommandLoader(_searchPath, _commandLoader);
		}

		[TearDown]
		public void TearDown()
		{
			_searchPath.Refresh();
			if (_searchPath.Exists)
			{
				_searchPath.Delete(true);
			}
		}


		[Test]
		public void LoadsAllAvailableCommands()
		{
			_commandLoader.Expect(x => x.LoadAll(_searchPath)).Return(new List<Assembly> { GetType().Assembly });
			_mocks.ReplayAll();

			List<ICommand> loadedCommands = _loader.LoadAll();
			Assert.AreEqual(2, loadedCommands.Count);
		}
	}


	public class MyTestCommand : ICommand
	{
		public string Identifier
		{
			get { return GetType().Name; }
		}

		public string Execute(ICommandAccess access, string[] arguments)
		{
			return "TEST";
		}
	}


	// For testing of abstract filtering. The command loader needs to exclude abstract classes.
	public abstract class MyAbstractCommand : ICommand
	{
		public virtual string Identifier
		{
			get { return GetType().Name; }
		}

		public abstract string Execute(ICommandAccess access, string[] arguments);
	}

	// This should be loaded since it inherits from the abstract class...
	public class MyUberCommand : MyAbstractCommand
	{
		public override string Execute(ICommandAccess access, string[] arguments)
		{
			return "";
		}
	}
}
