﻿using System;
using System.IO;
using System.Xml.Linq;

namespace Commander.Core
{
	public class ApplicationSettingsManager
	{
		private readonly FileInfo _settingsFile;

		public ApplicationSettingsManager()
		{
			_settingsFile = GetSettingsFile();
		}

		private static FileInfo GetSettingsFile()
		{
			string fileName = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "Commander");
			if (!Directory.Exists(fileName))
			{
				Directory.CreateDirectory(fileName);
			}

			fileName = Path.Combine(fileName, "Application Settings.xml");
			return new FileInfo(fileName);
		}


		public TabSettings LoadTabSettings()
		{
			var retVal = new TabSettings();
			if (!_settingsFile.Exists)
			{
				return retVal;
			}

			XDocument document = ReadDocument();
			XElement root = (XElement) document.FirstNode;
			XElement element = root.Element("TabSettings");
			return element != null ? retVal.Parse(element) : retVal;
		}

		public void WriteTabSettings(TabSettings tabSettings)
		{
			XDocument document = ReadDocument();
			XElement root = (XElement)document.FirstNode;
			XElement element = root.Element("TabSettings") ?? new XElement("TabSettings");

			if (element.Parent != null)
			{
				element.Remove();
			}

			element.RemoveAll();
			tabSettings.Write(element);

			root.Add(element);
			document.Save(_settingsFile.FullName);
		}

		private XDocument ReadDocument()
		{
			XDocument document;
			try
			{
				document = XDocument.Load(_settingsFile.FullName);
			}
			catch (Exception)
			{
				document = new XDocument();
				document.Add(new XElement("Settings"));
			}

			return document;
		}
	}
}
