﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace Commander.Core
{
	/// <summary>
	/// 
	/// </summary>
	public class ClipboardHistoryManager : IClipboardHistory
	{

		/// <summary>
		/// Initializes a new instance of the <see cref="ClipboardHistoryManager"/> class.
		/// </summary>
		public ClipboardHistoryManager()
		{
			Records = new List<string>(10);
		}

		/// <summary>
		/// Adds the specified data.
		/// </summary>
		/// <param name="data">The data.</param>
		public void Add(string data)
		{
			if (String.IsNullOrEmpty(data))
			{
				return;
			}

			if (NewItemSameAsLast(data))
			{
				return;
			}

			Records.Insert(0, data);
		}

		/// <summary>
		/// News the item same as last.
		/// </summary>
		/// <param name="data">The data.</param>
		/// <returns></returns>
		private bool NewItemSameAsLast(string data)
		{
			if (Records.Count < 1)
			{
				return false;
			}

			return Records[0].Equals(data, StringComparison.InvariantCulture);
		}

		/// <summary>
		/// Gets the content of the current clipboard.
		/// </summary>
		/// <value>
		/// The content of the current clipboard.
		/// </value>
		public string CurrentClipboardContent
		{
			get { return Clipboard.GetText(); }
		}

		/// <summary>
		/// Gets the records.
		/// </summary>
		public List<string> Records { get; private set; }
	}
}
