﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading;
using Commander.API;

namespace Commander.Core.Command
{
	public class CommandHost
	{
		private readonly ICommandManager _commandManager;
		private readonly ICommandLoader _commandLoader;
		public List<ICommand> LoadedCommands { get; private set; }

		public event EventHandler<EventArgs> CommandsLoaded;
		public event EventHandler<CommandLoadErrorArgs> CommandsLoadError;

		public CommandHost(ICommandManager commandManager, ICommandLoader commandLoader)
		{
			_commandManager = commandManager;
			_commandLoader = commandLoader;
			LoadedCommands = new List<ICommand>(10);
		}


		public void Start()
		{
			var t = new Thread(Load) {
			    Name = "Command.Loader", 
				IsBackground = true, 
				Priority = ThreadPriority.BelowNormal
			};

			t.Start();
		}

		private void Load()
		{
			try
			{
				LoadedCommands = _commandLoader.LoadAll();
				LoadedCommands.ForEach(_commandManager.RegisterHandler);

				TriggerLoadedEvent();
			}
			catch (Exception e)
			{
				Trace.WriteLine("Unable to complete the loading of commands: ");
				Trace.WriteLine(e);

				TriggerErrorEvent(e);
			}
		}

		private void TriggerErrorEvent(Exception exception)
		{
			if (CommandsLoadError != null)
			{
				CommandsLoadError.Invoke(this, new CommandLoadErrorArgs(exception));
			}
		}

		private void TriggerLoadedEvent()
		{
			if (CommandsLoaded != null)
			{
				CommandsLoaded.Invoke(this, EventArgs.Empty);
			}
		}
	}
}
