﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using Commander.API;

namespace Commander.Core.Command
{
	/// <summary>
	/// 
	/// </summary>
	public class CommandLoader : ICommandLoader
	{
		private readonly DirectoryInfo _searchPath;
		private readonly ICommandAssemblyLoader _assemblyLoader;


		public CommandLoader(DirectoryInfo searchPath)
			: this(searchPath, new CommandAssemblyLoader())
		{ }

		public CommandLoader(DirectoryInfo searchPath, ICommandAssemblyLoader assemblyLoader)
		{
			_searchPath = searchPath;
			_assemblyLoader = assemblyLoader;
		}


		public List<ICommand> LoadAll()
		{
			List<ICommand> retVal = new List<ICommand>(10);

			List<Assembly> commandAssemblies = _assemblyLoader.LoadAll(_searchPath);
			foreach (Assembly assembly in commandAssemblies)
			{
				retVal.AddRange(LoadCommands(assembly));
			}

			return retVal;
		}

		private static IEnumerable<ICommand> LoadCommands(Assembly assembly)
		{
			List<ICommand> retVal = new List<ICommand>(5);

			Module[] modules = assembly.GetModules(false);
			foreach (Module module in modules)
			{
				retVal.AddRange(HandleModule(module));
			}

			return retVal;
		}

		private static IEnumerable<ICommand> HandleModule(Module module)
		{
			IList<Type> types = module.GetTypes()
				.Where(t => t.IsPublic && 
					   !t.IsAbstract).ToList();

			return types.Select(Activator.CreateInstance).OfType<ICommand>();
		}
	}
}
