﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Commander.API;

namespace Commander.Core.Command
{
	public class CommandManager : ICommandManager
	{
		private readonly ICommandAccess _commandAccess;
		private readonly Dictionary<string, ICommand> _commandHandlers;

		public CommandManager(ICommandAccess commandAccess)
		{
			_commandAccess = commandAccess;
			_commandHandlers = new Dictionary<string, ICommand>(5);
		}

		public void RegisterHandler(ICommand executor)
		{
			_commandHandlers[executor.Identifier] = executor;
		}

		public bool Execute(string command)
		{
			string cmdName = command;
			List<string> myArgs = new List<string>();

			int spaceMarker = command.IndexOf(' ');
			if (spaceMarker > 0)
			{
				cmdName = command.Substring(0, spaceMarker);
				int start = spaceMarker;
				bool escapeActive = false;
				while (++spaceMarker < command.Length)
				{
					if (command[spaceMarker] == '"' || command[spaceMarker] == ' ' || spaceMarker > command.Length - 2)
					{
						if (command[spaceMarker] == '"')
						{
							escapeActive = !escapeActive;
						}

						if (spaceMarker > command.Length - 2)
						{
							++spaceMarker;
							escapeActive = false;
						}

						if (escapeActive)
						{
							continue;
						}

						string arg = command.Substring(start, spaceMarker - start).Trim('-', ' ', '"');
						if (!String.IsNullOrEmpty(arg))
						{
							myArgs.Add(arg);
						}
						start = spaceMarker + 1;
					}
				}
			}

			ICommand handler;
			if (!_commandHandlers.TryGetValue(cmdName, out handler))
			{
				return FinalCall(_commandAccess, cmdName);
			}

			var result = handler.Execute(_commandAccess, myArgs.ToArray());
			if (!String.IsNullOrEmpty(result))
			{
				_commandAccess.AppendOutput(Environment.NewLine + result, true, true);
			}

			return true;
		}

		private static bool FinalCall(ICommandAccess commandAccess, string command)
		{
			if (String.IsNullOrEmpty(command))
			{
				return false;
			}

			return HandleChangeDriveCommand(command, commandAccess);
		}

		private static bool HandleChangeDriveCommand(string command, ICommandAccess commandAccess)
		{
			command = command.Trim();

			// Test for a drive path only such as C: or D:)
			if (command.Length <= 3 && (command.EndsWith(":") || command.EndsWith("\\")))
			{
				DriveInfo[] drives = DriveInfo.GetDrives();
				var driveInfo = drives.Where(x => x.Name.StartsWith(command)).FirstOrDefault();
				if (driveInfo != null)
				{
					commandAccess.SetDirectory(driveInfo.RootDirectory.FullName);
					commandAccess.AppendOutput("", true, true);
					return true;
				}
			}

			return false;
		}
	}
}
