﻿using System;
using System.Drawing;
using System.Globalization;
using System.Xml.Linq;
using Commander.Core.Utils;

namespace Commander.Core
{
	public class TabSettings
	{
		public Font Font { get; set; }
		public Color TextColor { get; set; }
		public Color BackgroundColor { get; set; }
		public Color PromptColor { get; set; }

		public TabSettings()
			: this(new Font("Consolas", 11.5F, GraphicsUnit.Point), Color.Black, Color.Black, Color.White)
		{ }

		public TabSettings(Font font, Color promptColor, Color textColor, Color backgroundColor)
		{
			Font = font;
			TextColor = textColor;
			BackgroundColor = backgroundColor;
			PromptColor = promptColor;
		}

		public TabSettings Parse(XElement element)
		{
			XElement fontElement = element.Element("Font");
			if (fontElement != null)
			{
				FontStyle fontStyle = FontStyle.Regular;
				try
				{
					fontStyle = (FontStyle)Enum.Parse(typeof(FontStyle), fontElement.GetAttribute("style", "Regular"), true);
				}
				catch (Exception)
				{ }

				float emSize;
				if (!float.TryParse(fontElement.GetAttribute("size", "11.0"), NumberStyles.Float, CultureInfo.InvariantCulture, out emSize))
				{
					emSize = 11.0F;
				}

				Font = new Font(fontElement.GetAttribute("name", "Consolas"), 
							    emSize,
								fontStyle,
								GraphicsUnit.Point);
			}

			BackgroundColor = ParseColor(element.Element("Background"), Color.White);
			TextColor = ParseColor(element.Element("Text"), Color.Black);
			PromptColor = ParseColor(element.Element("Prompt"), Color.Black);

			return this;
		}

		private static Color ParseColor(XElement element, Color defaultColor)
		{
			if (element != null)
			{
				return Color.FromArgb(Convert.ToInt32(element.GetAttribute("color", Color.White.ToArgb().ToString())));
			}

			return defaultColor;
		}

		public void Write(XElement element)
		{
			element.Add(new XElement("Font", new XAttribute("name", Font.Name),
											 new XAttribute("size", Font.SizeInPoints.ToString(CultureInfo.InvariantCulture)),
											 new XAttribute("style", Font.Style.ToString())));
			element.Add(new XElement("Background", new XAttribute("color", BackgroundColor.ToArgb().ToString())));
			element.Add(new XElement("Prompt", new XAttribute("color", PromptColor.ToArgb().ToString())));
			element.Add(new XElement("Text", new XAttribute("color", TextColor.ToArgb().ToString())));
		}
	}
}
