﻿using System;
using System.Windows.Forms;
using Commander.Core.Command;

namespace Commander.UI.Presenters
{
	public class CommandLoadStatusViewPresenter : IPresenter<ToolStripStatusLabel>
	{
		private readonly StatusStrip _statusStrip;
		private readonly CommandHost _commandHost;

		public CommandLoadStatusViewPresenter(StatusStrip statusStrip, CommandHost commandHost, ToolStripStatusLabel statusLabel)
		{
			_statusStrip = statusStrip;
			_commandHost = commandHost;
			View = statusLabel;
		}

		private void OnCommandsLoaded(object sender, EventArgs e)
		{
			_statusStrip.Invoke(new Action(() => {
				CommandHost host = sender as CommandHost;
				View.Text = String.Format("{0} command extensions loaded", host.LoadedCommands.Count);
				View.Image = Resources.button_ok;
			}));
		}

		private void OnCommandLoadError(object sender, CommandLoadErrorArgs e)
		{
			_statusStrip.Invoke(new Action(() => {
				View.Text = "Error loading command extensions.";
				View.ToolTipText = e.Exception.Message;
				View.Image = Resources.status_unknown;
			}));
		}

		public void Init()
		{
			View.Image = Resources.button_ok;
			View.Text = "Loading...";
			View.ToolTipText = "Loading commands...";

			_commandHost.CommandsLoaded += OnCommandsLoaded;
			_commandHost.CommandsLoadError += OnCommandLoadError;
			_commandHost.Start();
		}

		public void Show()
		{
			View.Visible = true;
		}

		public void Hide()
		{
			View.Visible = false;
		}

		public bool IsVisible
		{
			get { return View != null && View.Visible; }
		}

		public ToolStripStatusLabel View { get; private set; }
	}
}
