﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Commander.Core;
using Commander.UI.Controls.Text;

namespace Commander.UI.Presenters
{
	/// <summary>
	/// 
	/// </summary>
	public class CommandWindowPopupMenuPresenter : IPresenter<ContextMenuStrip>
	{
		private readonly CommanderTextBox _target;
		private readonly IClipboardActions _clipboardActionReceiver;
		private readonly IClipboardHistory _clipboardHistory;
		private readonly ContextMenuStrip _view;
		private ToolStripMenuItem _copyItem;
		private ToolStripMenuItem _pasteItem;
		private ToolStripMenuItem _historyItem;

		public CommandWindowPopupMenuPresenter(CommanderTextBox target, IClipboardActions clipboardActionReceiver, 
											   IClipboardHistory clipboardHistory)
		{
			_target = target;
			_clipboardActionReceiver = clipboardActionReceiver;
			_clipboardHistory = clipboardHistory;
			_view = new ContextMenuStrip();
		}

		private void SyncClipboardHistory()
		{
			_historyItem.DropDownItems.Clear();

			List<string> records = _clipboardHistory.Records;
			_historyItem.Visible = (records.Count > 0);
			foreach (var item in records)
			{
				ToolStripMenuItem mi = new ToolStripMenuItem(item) {
				    TextAlign = ContentAlignment.MiddleLeft,
					ToolTipText = item
				};
				mi.Click += (sender, e) => _clipboardActionReceiver.Paste(ProcessClipboardItem(item));
				_historyItem.DropDownItems.Add(mi);
			}

			string current = _clipboardHistory.CurrentClipboardContent.Trim();
			_pasteItem.Enabled = !String.IsNullOrEmpty(current);
			_historyItem.Enabled = _historyItem.HasDropDownItems || _pasteItem.Enabled;
		}

		private static string ProcessClipboardItem(string item)
		{
			return item.Replace("\n", "").Replace("\r", "");
		}

		public void Init()
		{
			_copyItem = new ToolStripMenuItem("Copy");
			_pasteItem = new ToolStripMenuItem("Past");
			_historyItem = new ToolStripMenuItem("History");
			_view.Items.Add(_copyItem);
			_view.Items.Add(_pasteItem);
			_view.Items.Add(new ToolStripSeparator());
			_view.Items.Add(_historyItem);

			_copyItem.Click += (sender, e) => _clipboardActionReceiver.Copy();
			_pasteItem.Click += (sender, e) => _clipboardActionReceiver.Paste();
			_target.ContextMenuStrip = _view;

			_copyItem.Enabled = false;
			_target.SelectionChanged += OnTargetControlSelectionChanged;

			_historyItem.Click += OnShowHistoryClicked;
			
			_view.VisibleChanged += OnMenuVisibleChanged;
		}

		private void OnShowHistoryClicked(object sender, EventArgs e)
		{
			using (ClipboardHistoryPresenter p = new ClipboardHistoryPresenter(_clipboardHistory, _clipboardActionReceiver, _view.FindForm()))
			{
				p.Init();
				p.Show();
			}
		}

		private void OnMenuVisibleChanged(object sender, EventArgs e)
		{
			if (_view.Visible)
			{
				SyncClipboardHistory();
			}
		}

		private void OnTargetControlSelectionChanged(object sender, EventArgs e)
		{
			_copyItem.Enabled = !String.IsNullOrEmpty(_target.SelectedText);
		}

		public void Show()
		{
			_view.Visible = true;
		}

		public void Hide()
		{
			_view.Visible = false;
		}

		public bool IsVisible
		{
			get { return _view.Visible; }
		}

		public ContextMenuStrip View
		{
			get { return _view; }
		}
	}
}
