﻿using System;
using System.Windows.Forms;
using Commander.API;
using Commander.UI.Controls.Text;

namespace Commander.UI.Presenters
{
	public class CommanderDragDropPresenter : IPresenter<CommanderTextBox>
	{
		private readonly CommanderTextBox _view;
		private readonly IOutput _output;

		public CommanderDragDropPresenter(CommanderTextBox view, IOutput output)
		{
			_view = view;
			_output = output;
		}

		public void Init()
		{
			_view.AllowDrop = true;
			_view.EnableAutoDragDrop = false;
			_view.DragDrop += OnDragDrop;
			_view.DragEnter += OnDragDropStart;
		}

		private static void OnDragDropStart(object sender, DragEventArgs e)
		{
			if (e.Data.GetDataPresent(DataFormats.FileDrop) || e.Data.GetDataPresent(DataFormats.StringFormat))
			{
				e.Effect = DragDropEffects.Copy;
			}
		}

		private void OnDragDrop(object sender, DragEventArgs e)
		{
			if (e.Data.GetDataPresent(DataFormats.FileDrop))
			{
				string[] data = (string[])e.Data.GetData(DataFormats.FileDrop);
				if (data.Length > 0)
				{
					string path = data[0];
					if (path.Contains(" "))
					{
						path = String.Format("\"{0}\"", path);
					}
					_output.AppendOutput(path, false, false);
				}
			}
			else if (e.Data.GetDataPresent(DataFormats.StringFormat))
			{
				string data = (string)e.Data.GetData(DataFormats.StringFormat);
				if (!String.IsNullOrEmpty(data))
				{
					_output.AppendOutput(data, false, false);
				}
			}
		}

		public void Show()
		{ }

		public void Hide()
		{ }

		public bool IsVisible
		{
			get { return _view.Visible; }
		}

		public CommanderTextBox View
		{
			get { return _view; }
		}
	}
}
